@@ -1,8 +1,7 @@
 use crate::interactive::{
     app::tree_view::TreeView,
-    app::FocussedPane::*,
-    widgets::{GlobPane, HelpPane, MainWindow, MarkMode, MarkPane},
-    AppState, DisplayOptions, EntryDataBundle,
+    widgets::{Column, GlobPane, HelpPane, MainWindow, MarkMode, MarkPane},
+    DisplayOptions, EntryDataBundle,
 };
 use crosstermion::input::Key;
 use dua::traverse::TreeIndex;
@@ -10,6 +9,8 @@ use std::{fs, io, path::PathBuf};
 use tui::backend::Backend;
 use tui_react::Terminal;
 
+use super::state::{AppState, FocussedPane::*};
+
 #[derive(Copy, Clone)]
 pub enum CursorMode {
     Advance,
@@ -134,6 +135,22 @@ impl AppState {
         self.entries = tree_view.sorted_entries(self.navigation().view_root, self.sorting);
     }
 
+    pub fn toggle_mtime_column(&mut self) {
+        self.toggle_column(Column::MTime);
+    }
+
+    pub fn toggle_count_column(&mut self) {
+        self.toggle_column(Column::Count);
+    }
+
+    fn toggle_column(&mut self, column: Column) {
+        if self.show_columns.contains(&column) {
+            self.show_columns.remove(&column);
+        } else {
+            self.show_columns.insert(column);
+        }
+    }
+
     pub fn toggle_glob_search(&mut self, window: &mut MainWindow) {
         self.focussed = match self.focussed {
             Main | Mark | Help => {
@@ -145,7 +162,7 @@ impl AppState {
     }
 
     pub fn reset_message(&mut self) {
-        if self.is_scanning {
+        if self.scan.is_some() {
             self.message = Some("-> scanning <-".into());
         } else {
             self.message = None;
@@ -295,7 +312,8 @@ impl AppState {
         let parent_idx = tree_view
             .fs_parent_of(index)
             .expect("us being unable to delete the root index");
-        let entries_deleted = tree_view.remove_entries(index);
+        let entries_deleted =
+            tree_view.remove_entries(index, true /* remove node at `index` */);
 
         if !tree_view.exists(self.navigation().view_root) {
             self.go_to_root(tree_view);
@@ -317,7 +335,7 @@ impl AppState {
         entries_deleted
     }
 
-    fn go_to_root(&mut self, tree_view: &TreeView<'_>) {
+    pub fn go_to_root(&mut self, tree_view: &TreeView<'_>) {
         let root = self.navigation().tree_root;
         let entries = tree_view.sorted_entries(root, self.sorting);
         self.navigation_mut().exit_node(root, &entries);
